/* fillv2.c
 *------------------------------------------------------------------------------
 * Fill synoptic map arrays.
 *
 * fillEmap ()
 * fillWmap ()
 * fillCmap ()
 * fillEfile ()
 * fillWfile ()
 *------------------------------------------------------------------------------
 * Author:  Alice Lecinski   HAO/NCAR    3 Nov 1997
 * Updates: Andrew Stanger   HAO/NCAR
 *  8 Nov 2002: First row of map is SOUTH, instead of north (previous versions).
 *------------------------------------------------------------------------------
 */
#include <stdio.h>
#include <ctype.h>
#include "EWmap4.h"

/* short BCmap [PA][BLONGS]; */	/* Position angle goes from north to south to
				 * north, 0 to 359 degrees.
				 *
				 * BCmap longitude index "goes" fastest.
				 */

/* short Bmap [MLATS][BLONGS];*/ /* East and West maps go from south to north.
				  * Bmap longitude index "goes" fastest.
				  */

/* For both Bmap and BCmap, (imap and cmap) 
 *     longitude goes from 0 to 360 degrees, 
 *     with 0 degrees on the left and 360 degrees on the right.
 *     Time is reversed, so that the earliest day is on the right, 
 *     and the latest day is on the left.
 *
 * In idl, cmap can be displayed with:	a = INTARR (360, 360)
 *					TVSCL, a, order = 0
 *
 * In idl, imap can be displayed with:	a = INTARR (360, 181)
 *					TVSCL, a, order = 0
 */

/* float pB [DAYS][LATS] ; */	/* East file goes from north to south
				 * West file goes from south to north
				 *
				 * Time goes in the forward direction.
				 * pB latitude index "goes" fastest.
				 */
extern int offset ;

double  El_beg = 2445061 , El_lconst = 2445335 , El_last = 2446796 ;
float   El_corr = 75 , El_slope = 0.051335 ;
float   El_fix, get_El_fix () ;

/*
 * pB units are 10^-10 B/Bsun for input data and for Bmap and imap.
 * pB units are 10^-10 B/Bsun for input data and for BCmap and cmap.
 * pB units for the EWfile are are 10^-8 B/Bsun.
 */

/*----------------------------------------------------------------------------*/

void
fillEmap (bigmap, azbuf, flong, f_juld) 

    short *bigmap ;			/* Big synoptic map array. */
    short *azbuf ;			/* Azimuthal scan array. */
    float flong ;
    double f_juld ;
{
    int ilat, ilong ;
    short *big, *azp ;  

    El_fix = get_El_fix (f_juld) ;	/* El Chicon fix. */

    fprintf (stdout, "El_fix: %10.4f\n", El_fix) ;

    ilong = (int) flong ;

    azp = azbuf ;
    azp += 180 ;			/* Start at position angle 180. */
    big = bigmap ;

    /*** Fill Bmap South to North ***/
    /*   bigmap [ilat][ilong+offset] = azbuf [180-ilat]   */

    for ( ilat = 0 ; ilat < MLATS ;  ilat++ )
        {
	*(big + ilong + offset + ilat * BLONGS) = *azp + (short) El_fix ;
	azp-- ;
        }

}

/*----------------------------------------------------------------------------*/

void
fillWmap (bigmap, azbuf, flong, f_juld) 

    short *bigmap ;			/* Big synoptic map array. */
    short *azbuf ;			/* Azimuthal scan array. */
    float flong ;
    double f_juld ;
{
    int ilat , ilong ;
    short *azp , *big ;

    El_fix = get_El_fix (f_juld) ;	/* El Chicon fix. */

    ilong = (int) flong ;

    big = bigmap  ;
    azp = azbuf ;
    ilat = 180 ;

    /*** Fill in value at North Pole. ***/

    ilat = 180 ;
    *(bigmap + ilong + offset + ilat * BLONGS) = *azp + (short) El_fix ;

    azp += 180 ;			/* Start at position angle 180. */

    /*** Fill Bmap South to North ***/
    /*   bigmap [ilat][ilong+offset] = azbuf [ilat+180]   */

    for ( ilat = 0 ; ilat < MLATS - 1 ; ilat++ )
	{
	*(big + ilong + offset + ilat * BLONGS) = *azp + (short) El_fix ;
	azp++ ;
        }
    fprintf (stdout, "fillWmap  done\n") ;
    fflush  (stdout) ;
}

/*----------------------------------------------------------------------------*/

void
fillCmap (bigmap, azbuf, flong, f_juld) 

    short *bigmap ;			/* Big synoptic map array. */
    short *azbuf ;			/* Azimuthal scan array. */
    float flong ;
    double f_juld ;
{
    int pa , ilong ;
    short *azp , *big  ;  

    El_fix = get_El_fix (f_juld) ;	/* El Chicon fix. */
    ilong = (int) flong ;

    azp = azbuf ;
    big = bigmap ;
    for ( pa = 0 ; pa < PA ;  pa++ )
	{				/* Fill BCmap North to South to North */
	*(big + pa * BLONGS + ilong + offset) = *azp + (short) El_fix ;
	azp++ ;
        }
}

/*----------------------------------------------------------------------------*/

void
fillEfile (fpB, azbuf, iday)

float *fpB ;
short *azbuf ;				/* Azimuthal scan array. */
int iday ;
{
    int  ll , kk ;
    short *azp  ;  
    float  avg3 ;  
    float  avg5 ;  

    /*** pB data for east file goes from north to south. ***/

    azp = azbuf ;

    ll = 0 ;
	{
	for ( kk = 0, avg3 = 0 ; kk < 3 ;  kk++ )
	    { avg3 += (float) *azp ; azp++ ; }
	avg3 /= 300.0 ;			/* Avg and convert to 10^-8 B/Bsun */
	*(fpB + ll + iday * LATS) = avg3 ;	/* fpb [iday][0] = avg3 */
        }

    for ( ll = 1 ; ll < 36 ; ll++ )
	{
	for ( kk = 0, avg5 = 0 ; kk < 5 ;  kk++ )
	    { avg5 += (float) *azp ; azp++ ; }
	avg5 /= 500.0 ;			/* Avg and convert to 10^-8 B/Bsun */
	*(fpB + ll + iday * LATS) = avg5 ;	/* fpb [iday][ll] = avg5 */
        }

    ll = 36 ;
	{
	for ( kk = 0, avg3 = 0 ; kk < 3 ;  kk++ )
	    { avg3 += (float) *azp ; azp++ ; }
	avg3 /= 300.0 ;			/* Avg and convert to 10^-8 B/Bsun */
	*(fpB + ll + iday * LATS) = avg3 ;	/* fpb [iday][36] = avg3 */
        }
}

/*----------------------------------------------------------------------------*/

void
fillWfile (fpB, azbuf, iday)

float *fpB ;
short *azbuf ;				/* Azimuthal scan array. */
int iday ;
{
    int ll , kk ;
    short *azp  ;  
    float avg2 ;  
    float avg3 ;  
    float avg5 ;  

    /*** pB data for west file goes from south to north. ***/

    azp = azbuf ;
    azp += 180 ;

    ll = 0 ;
	{
	for ( kk = 0, avg3 = 0 ; kk < 3 ; kk++ )
	    { avg3 += (float) *azp ; azp++ ; }
	avg3 /= 300.0 ;			/* Avg and convert to 10^-8 B/Bsun */
	*(fpB + ll + iday * LATS) = avg3 ;	/* fpb [iday][0] = avg3 */
        }

    for ( ll = 1 ; ll < 36 ; ll++ )
	{
	for ( kk = 0, avg5 = 0 ; kk < 5 ; kk++ )
	    { avg5 += (float) *azp ; azp++ ; }
	avg5 /= 500.0 ;			/* Avg and convert to 10^-8 B/Bsun */
	*(fpB + ll + iday * LATS) = avg5 ;	/* fpb [iday][ll] = avg5 */
        }

    ll = 36 ;
	{
	for ( kk = 0, avg3 = 0 ; kk < 3 ; kk++ )
	    { avg3 += (float) *azp ; azp++ ; }
	azp = azbuf ;
	avg3 += (float) *azp ;
	avg3 /= 300.0 ;			/* Avg and convert to 10^-8 B/Bsun */
	*(fpB + ll + iday * LATS) = avg3 ;	/* fpb [iday][36] = avg3 */
        }
}

/*----------------------------------------------------------------------------*/

float
get_El_fix (f_juld)

double f_juld ;
{
    /*** El Chicon fix in terms of 10^-10 B/Bsun ***/

    if ( f_juld >= El_beg && f_juld <= El_last )
	{
	if ( f_juld <= El_lconst ) 
		return ( El_corr ) ;
	else
		return ( El_corr - El_slope * (f_juld - El_lconst) ) ;
        }
    else
	{
        return ( 0 ) ;
        }
}

/*----------------------------------------------------------------------------*/

